﻿#include "object-enchant/smith-tables.h"
#include "artifact/random-art-effects.h"
#include "object-enchant/object-smith.h"
#include "object-enchant/smith-info.h"
#include "object-enchant/smith-types.h"
#include "object-enchant/tr-flags.h"
#include "object-enchant/tr-types.h"

#include <memory>
#include <unordered_map>
#include <vector>

/*!
 * @brief エッセンスの順序リスト。エッセンスの表示順等で使用する。
 */
const std::vector<SmithEssence> Smith::essence_list_order = {
    SmithEssence::STR,
    SmithEssence::INT,
    SmithEssence::WIS,
    SmithEssence::DEX,
    SmithEssence::CON,
    SmithEssence::CHR,

    SmithEssence::SUST_STATUS,

    SmithEssence::MAGIC_MASTERY,
    SmithEssence::STEALTH,
    SmithEssence::SEARCH,
    SmithEssence::INFRA,
    SmithEssence::TUNNEL,
    SmithEssence::SPEED,
    SmithEssence::BLOWS,

    SmithEssence::CHAOTIC,
    SmithEssence::VAMPIRIC,
    SmithEssence::EATHQUAKE,
    SmithEssence::BRAND_POIS,
    SmithEssence::BRAND_ACID,
    SmithEssence::BRAND_ELEC,
    SmithEssence::BRAND_FIRE,
    SmithEssence::BRAND_COLD,
    SmithEssence::BRAND_MAGIC,

    SmithEssence::IMMUNITY,
    SmithEssence::REFLECT,

    SmithEssence::RES_ACID,
    SmithEssence::RES_ELEC,
    SmithEssence::RES_FIRE,
    SmithEssence::RES_COLD,
    SmithEssence::RES_POIS,
    SmithEssence::RES_FEAR,
    SmithEssence::RES_LITE,
    SmithEssence::RES_DARK,
    SmithEssence::RES_BLIND,
    SmithEssence::RES_CONF,
    SmithEssence::RES_SOUND,
    SmithEssence::RES_SHARDS,
    SmithEssence::RES_NETHER,
    SmithEssence::RES_NEXUS,
    SmithEssence::RES_CHAOS,
    SmithEssence::RES_DISEN,
    SmithEssence::RES_WATER,
    SmithEssence::RES_TIME,
    SmithEssence::RES_CURSE,

    SmithEssence::HOLD_EXP,
    SmithEssence::FREE_ACT,
    SmithEssence::WARNING,
    SmithEssence::LEVITATION,
    SmithEssence::SEE_INVIS,
    SmithEssence::SLOW_DIGEST,
    SmithEssence::REGEN,
    SmithEssence::TELEPORT,
    SmithEssence::NO_MAGIC,
    SmithEssence::LITE,
    SmithEssence::NO_TELE,
    SmithEssence::TELEPATHY,

    SmithEssence::SLAY_EVIL,
    SmithEssence::SLAY_ANIMAL,
    SmithEssence::SLAY_UNDEAD,
    SmithEssence::SLAY_DEMON,
    SmithEssence::SLAY_ORC,
    SmithEssence::SLAY_TROLL,
    SmithEssence::SLAY_GIANT,
    SmithEssence::SLAY_DRAGON,
    SmithEssence::SLAY_HUMAN,
    SmithEssence::SLAY_GOOD,

    SmithEssence::EASY2_WEAPON,
    SmithEssence::STRENGTHEN_BOW,
    SmithEssence::UNIQUE,

    SmithEssence::ATTACK,
    SmithEssence::AC,
};

/*!
 * @brief SmithEssence からエッセンスの表記名を引く連想配列
 */
const std::unordered_map<SmithEssence, concptr> Smith::essence_to_name = {
    { SmithEssence::STR, _("腕力", "strength") },
    { SmithEssence::INT, _("知能", "intelligen.") },
    { SmithEssence::WIS, _("賢さ", "wisdom") },
    { SmithEssence::DEX, _("器用さ", "dexterity") },
    { SmithEssence::CON, _("耐久力", "constitut.") },
    { SmithEssence::CHR, _("魅力", "charisma") },

    { SmithEssence::SUST_STATUS, _("能力維持", "sustain") },

    { SmithEssence::MAGIC_MASTERY, _("魔力支配", "magic mast.") },
    { SmithEssence::STEALTH, _("隠密", "stealth") },
    { SmithEssence::SEARCH, _("探索", "searching") },
    { SmithEssence::INFRA, _("赤外線視力", "infravision") },
    { SmithEssence::TUNNEL, _("採掘", "digging") },
    { SmithEssence::SPEED, _("スピード", "speed") },
    { SmithEssence::BLOWS, _("追加攻撃", "extra atk") },

    { SmithEssence::CHAOTIC, _("カオス攻撃", "chaos brand") },
    { SmithEssence::VAMPIRIC, _("吸血攻撃", "vampiric") },
    { SmithEssence::EATHQUAKE, _("地震", "quake") },
    { SmithEssence::BRAND_POIS, _("毒殺", "pois. brand") },
    { SmithEssence::BRAND_ACID, _("溶解", "acid brand") },
    { SmithEssence::BRAND_ELEC, _("電撃", "elec. brand") },
    { SmithEssence::BRAND_FIRE, _("焼棄", "fire brand") },
    { SmithEssence::BRAND_COLD, _("凍結", "cold brand") },
    { SmithEssence::BRAND_MAGIC, _("魔術", "magic brand") },

    { SmithEssence::IMMUNITY, _("免疫", "immunity") },
    { SmithEssence::REFLECT, _("反射", "reflection") },

    { SmithEssence::RES_ACID, _("耐酸", "res. acid") },
    { SmithEssence::RES_ELEC, _("耐電撃", "res. elec.") },
    { SmithEssence::RES_FIRE, _("耐火炎", "res. fire") },
    { SmithEssence::RES_COLD, _("耐冷気", "res. cold") },
    { SmithEssence::RES_POIS, _("耐毒", "res. poison") },
    { SmithEssence::RES_FEAR, _("耐恐怖", "res. fear") },
    { SmithEssence::RES_LITE, _("耐閃光", "res. light") },
    { SmithEssence::RES_DARK, _("耐暗黒", "res. dark") },
    { SmithEssence::RES_BLIND, _("耐盲目", "res. blind") },
    { SmithEssence::RES_CONF, _("耐混乱", "res.confuse") },
    { SmithEssence::RES_SOUND, _("耐轟音", "res. sound") },
    { SmithEssence::RES_SHARDS, _("耐破片", "res. shard") },
    { SmithEssence::RES_NETHER, _("耐地獄", "res. nether") },
    { SmithEssence::RES_NEXUS, _("耐因果混乱", "res. nexus") },
    { SmithEssence::RES_CHAOS, _("耐カオス", "res. chaos") },
    { SmithEssence::RES_DISEN, _("耐劣化", "res. disen.") },
    { SmithEssence::RES_WATER, _("耐水", "res. water") },
    { SmithEssence::RES_TIME, _("耐時間逆転", "res. time") },
    { SmithEssence::RES_CURSE, _("耐呪力", "res. curse") },

    { SmithEssence::HOLD_EXP, _("経験値維持", "hold exp") },
    { SmithEssence::FREE_ACT, _("麻痺知らず", "free action") },
    { SmithEssence::WARNING, _("警告", "warning") },
    { SmithEssence::LEVITATION, _("浮遊", "levitation") },
    { SmithEssence::SEE_INVIS, _("可視透明", "see invis.") },
    { SmithEssence::TELEPATHY, _("テレパシー", "telepathy") },
    { SmithEssence::SLOW_DIGEST, _("遅消化", "slow dige.") },
    { SmithEssence::REGEN, _("急速回復", "regen.") },
    { SmithEssence::TELEPORT, _("テレポート", "teleport") },
    { SmithEssence::NO_MAGIC, _("反魔法", "anti magic") },
    { SmithEssence::LITE, _("永久光源", "perm. light") },
    { SmithEssence::NO_TELE, _("反テレポ", "no teleport") },

    { SmithEssence::SLAY_EVIL, _("邪悪倍打", "slay animal") },
    { SmithEssence::SLAY_ANIMAL, _("動物倍打", "slay evil") },
    { SmithEssence::SLAY_UNDEAD, _("不死倍打", "slay undead") },
    { SmithEssence::SLAY_DEMON, _("悪魔倍打", "slay demon") },
    { SmithEssence::SLAY_ORC, _("オーク倍打", "slay orc") },
    { SmithEssence::SLAY_TROLL, _("トロル倍打", "slay troll") },
    { SmithEssence::SLAY_GIANT, _("巨人倍打", "slay giant") },
    { SmithEssence::SLAY_DRAGON, _("竜倍打", "slay dragon") },
    { SmithEssence::SLAY_HUMAN, _("人間倍打", "slay human") },
    { SmithEssence::SLAY_GOOD, _("善良倍打", "slay good") },

    { SmithEssence::EASY2_WEAPON, _("二刀流", "two weapons") },
    { SmithEssence::STRENGTHEN_BOW, _("射撃強化", "enc. ranged") },
    { SmithEssence::UNIQUE, _("ユニーク", "unique") },

    { SmithEssence::ATTACK, _("攻撃", "weapon enc.") },
    { SmithEssence::AC, _("防御", "armor enc.") },
};

/*!
 * @brief エッセンス抽出情報テーブル
 */
const std::vector<essence_drain_type> Smith::essence_drain_info_table = {
    { TR_STR, { SmithEssence::STR }, 10 },
    { TR_INT, { SmithEssence::INT }, 10 },
    { TR_WIS, { SmithEssence::WIS }, 10 },
    { TR_DEX, { SmithEssence::DEX }, 10 },
    { TR_CON, { SmithEssence::CON }, 10 },
    { TR_CHR, { SmithEssence::CHR }, 10 },
    { TR_MAGIC_MASTERY, { SmithEssence::MAGIC_MASTERY }, 10 },
    { TR_FORCE_WEAPON, { SmithEssence::INT, SmithEssence::WIS }, 5 },
    { TR_STEALTH, { SmithEssence::STEALTH }, 10 },
    { TR_SEARCH, { SmithEssence::SEARCH }, 10 },
    { TR_INFRA, { SmithEssence::INFRA }, 10 },
    { TR_TUNNEL, { SmithEssence::TUNNEL }, 10 },
    { TR_SPEED, { SmithEssence::SPEED }, 10 },
    { TR_BLOWS, { SmithEssence::BLOWS }, 10 },
    { TR_CHAOTIC, { SmithEssence::CHAOTIC }, 10 },
    { TR_VAMPIRIC, { SmithEssence::VAMPIRIC }, 10 },
    { TR_SLAY_ANIMAL, { SmithEssence::SLAY_ANIMAL }, 10 },
    { TR_SLAY_EVIL, { SmithEssence::SLAY_EVIL }, 10 },
    { TR_SLAY_UNDEAD, { SmithEssence::SLAY_UNDEAD }, 10 },
    { TR_SLAY_DEMON, { SmithEssence::SLAY_DEMON }, 10 },
    { TR_SLAY_ORC, { SmithEssence::SLAY_ORC }, 10 },
    { TR_SLAY_TROLL, { SmithEssence::SLAY_TROLL }, 10 },
    { TR_SLAY_GIANT, { SmithEssence::SLAY_GIANT }, 10 },
    { TR_SLAY_DRAGON, { SmithEssence::SLAY_DRAGON }, 10 },
    { TR_KILL_DRAGON, { SmithEssence::SLAY_DRAGON }, 10 },
    { TR_VORPAL, { SmithEssence::BRAND_POIS, SmithEssence::BRAND_ACID, SmithEssence::BRAND_ELEC, SmithEssence::BRAND_FIRE, SmithEssence::BRAND_COLD }, 5 },
    { TR_EARTHQUAKE, { SmithEssence::EATHQUAKE }, 10 },
    { TR_BRAND_POIS, { SmithEssence::BRAND_POIS }, 10 },
    { TR_BRAND_ACID, { SmithEssence::BRAND_ACID }, 10 },
    { TR_BRAND_ELEC, { SmithEssence::BRAND_ELEC }, 10 },
    { TR_BRAND_FIRE, { SmithEssence::BRAND_FIRE }, 10 },
    { TR_BRAND_COLD, { SmithEssence::BRAND_COLD }, 10 },

    { TR_SUST_STR, { SmithEssence::SUST_STATUS }, 10 },
    { TR_SUST_INT, { SmithEssence::SUST_STATUS }, 10 },
    { TR_SUST_WIS, { SmithEssence::SUST_STATUS }, 10 },
    { TR_SUST_DEX, { SmithEssence::SUST_STATUS }, 10 },
    { TR_SUST_CON, { SmithEssence::SUST_STATUS }, 10 },
    { TR_SUST_CHR, { SmithEssence::SUST_STATUS }, 10 },
    { TR_RIDING, {}, 0 },
    { TR_EASY_SPELL, { SmithEssence::BRAND_MAGIC }, 10 },
    { TR_IM_ACID, { SmithEssence::IMMUNITY }, 100 },
    { TR_IM_ELEC, { SmithEssence::IMMUNITY }, 100 },
    { TR_IM_FIRE, { SmithEssence::IMMUNITY }, 100 },
    { TR_IM_COLD, { SmithEssence::IMMUNITY }, 100 },
    { TR_THROW, {}, 0 },
    { TR_REFLECT, { SmithEssence::REFLECT }, 10 },
    { TR_FREE_ACT, { SmithEssence::FREE_ACT }, 10 },
    { TR_HOLD_EXP, { SmithEssence::HOLD_EXP }, 10 },
    { TR_RES_ACID, { SmithEssence::RES_ACID }, 10 },
    { TR_RES_ELEC, { SmithEssence::RES_ELEC }, 10 },
    { TR_RES_FIRE, { SmithEssence::RES_FIRE }, 10 },
    { TR_RES_COLD, { SmithEssence::RES_COLD }, 10 },
    { TR_RES_POIS, { SmithEssence::RES_POIS }, 10 },
    { TR_RES_FEAR, { SmithEssence::RES_FEAR }, 10 },
    { TR_RES_LITE, { SmithEssence::RES_LITE }, 10 },
    { TR_RES_DARK, { SmithEssence::RES_DARK }, 10 },
    { TR_RES_BLIND, { SmithEssence::RES_BLIND }, 10 },
    { TR_RES_CONF, { SmithEssence::RES_CONF }, 10 },
    { TR_RES_SOUND, { SmithEssence::RES_SOUND }, 10 },
    { TR_RES_SHARDS, { SmithEssence::RES_SHARDS }, 10 },
    { TR_RES_NETHER, { SmithEssence::RES_NETHER }, 10 },
    { TR_RES_NEXUS, { SmithEssence::RES_NEXUS }, 10 },
    { TR_RES_CHAOS, { SmithEssence::RES_CHAOS }, 10 },
    { TR_RES_DISEN, { SmithEssence::RES_DISEN }, 10 },

    { TR_SH_FIRE, { SmithEssence::BRAND_FIRE, SmithEssence::RES_FIRE }, 10 },
    { TR_SH_ELEC, { SmithEssence::BRAND_ELEC, SmithEssence::RES_ELEC }, 10 },
    { TR_SLAY_HUMAN, { SmithEssence::SLAY_HUMAN }, 10 },
    { TR_SH_COLD, { SmithEssence::BRAND_COLD, SmithEssence::RES_COLD }, 10 },
    { TR_NO_TELE, { SmithEssence::NO_TELE }, 10 },
    { TR_NO_MAGIC, { SmithEssence::NO_MAGIC }, 10 },
    { TR_DEC_MANA, { SmithEssence::INT }, 10 },
    { TR_TY_CURSE, {}, -1 },
    { TR_WARNING, { SmithEssence::WARNING }, 10 },
    { TR_HIDE_TYPE, {}, 0 },
    { TR_SHOW_MODS, {}, 0 },
    { TR_SLAY_GOOD, { SmithEssence::SLAY_GOOD }, 10 },
    { TR_LEVITATION, { SmithEssence::LEVITATION }, 10 },
    { TR_LITE_1, { SmithEssence::LITE }, 10 },
    { TR_SEE_INVIS, { SmithEssence::SEE_INVIS }, 10 },
    { TR_TELEPATHY, { SmithEssence::TELEPATHY }, 10 },
    { TR_SLOW_DIGEST, { SmithEssence::SLOW_DIGEST }, 10 },
    { TR_REGEN, { SmithEssence::REGEN }, 10 },
    { TR_XTRA_MIGHT, { SmithEssence::STRENGTHEN_BOW }, 10 },
    { TR_XTRA_SHOTS, { SmithEssence::STRENGTHEN_BOW }, 10 },
    { TR_IGNORE_ACID, {}, 0 },
    { TR_IGNORE_ELEC, {}, 0 },
    { TR_IGNORE_FIRE, {}, 0 },
    { TR_IGNORE_COLD, {}, 0 },
    { TR_ACTIVATE, {}, 0 },
    { TR_DRAIN_EXP, {}, -1 },
    { TR_TELEPORT, { SmithEssence::TELEPORT }, 10 },
    { TR_AGGRAVATE, {}, -1 },
    { TR_BLESSED, {}, 0 },
    { TR_XXX_93, {}, 0 },
    { TR_XXX_94, {}, 0 },
    { TR_KILL_GOOD, { SmithEssence::SLAY_GOOD }, 10 },

    { TR_KILL_ANIMAL, { SmithEssence::SLAY_ANIMAL }, 10 },
    { TR_KILL_EVIL, { SmithEssence::SLAY_EVIL }, 10 },
    { TR_KILL_UNDEAD, { SmithEssence::SLAY_UNDEAD }, 10 },
    { TR_KILL_DEMON, { SmithEssence::SLAY_DEMON }, 10 },
    { TR_KILL_ORC, { SmithEssence::SLAY_ORC }, 10 },
    { TR_KILL_TROLL, { SmithEssence::SLAY_TROLL }, 10 },
    { TR_KILL_GIANT, { SmithEssence::SLAY_GIANT }, 10 },
    { TR_KILL_HUMAN, { SmithEssence::SLAY_HUMAN }, 10 },
    { TR_ESP_ANIMAL, { SmithEssence::SLAY_ANIMAL }, 10 },
    { TR_ESP_UNDEAD, { SmithEssence::SLAY_UNDEAD }, 10 },
    { TR_ESP_DEMON, { SmithEssence::SLAY_DEMON }, 10 },
    { TR_ESP_ORC, { SmithEssence::SLAY_ORC }, 10 },
    { TR_ESP_TROLL, { SmithEssence::SLAY_TROLL }, 10 },
    { TR_ESP_GIANT, { SmithEssence::SLAY_GIANT }, 10 },
    { TR_ESP_DRAGON, { SmithEssence::SLAY_DRAGON }, 10 },
    { TR_ESP_HUMAN, { SmithEssence::SLAY_HUMAN }, 10 },
    { TR_ESP_EVIL, { SmithEssence::SLAY_EVIL }, 10 },
    { TR_ESP_GOOD, { SmithEssence::SLAY_GOOD }, 10 },
    { TR_ESP_NONLIVING, {}, 0 }, // TODO
    { TR_ESP_UNIQUE, { SmithEssence::UNIQUE }, 20 },
    { TR_FULL_NAME, {}, 0 },
    { TR_FIXED_FLAVOR, {}, 0 },
    { TR_ADD_L_CURSE, {}, -1 },
    { TR_ADD_H_CURSE, {}, -1 },
    { TR_DRAIN_HP, {}, -1 },
    { TR_DRAIN_MANA, {}, -1 },
    { TR_LITE_2, { SmithEssence::LITE }, 20 },
    { TR_LITE_3, { SmithEssence::LITE }, 30 },
    { TR_LITE_M1, {}, 0 },
    { TR_LITE_M2, {}, 0 },
    { TR_LITE_M3, {}, 0 },
    { TR_LITE_FUEL, {}, 0 },

    { TR_CALL_ANIMAL, {}, -1 },
    { TR_CALL_DEMON, {}, -1 },
    { TR_CALL_DRAGON, {}, -1 },
    { TR_CALL_UNDEAD, {}, -1 },
    { TR_COWARDICE, {}, -1 },
    { TR_LOW_MELEE, {}, -1 },
    { TR_LOW_AC, {}, -1 },
    { TR_HARD_SPELL, {}, -1 },
    { TR_FAST_DIGEST, {}, -1 },
    { TR_SLOW_REGEN, {}, -1 },
    { TR_MIGHTY_THROW, {}, 0 },
    { TR_EASY2_WEAPON, { SmithEssence::EASY2_WEAPON }, 10 },
    { TR_DOWN_SAVING, {}, -1 },
    { TR_NO_AC, {}, -1 },
    { TR_HEAVY_SPELL, {}, -1 },
    { TR_RES_TIME, { SmithEssence::RES_TIME }, 10 },
    { TR_RES_WATER, { SmithEssence::RES_WATER }, 10 },
    { TR_INVULN_ARROW, {}, 0 },
    { TR_DARK_SOURCE, {}, 0 },
    { TR_SUPPORTIVE, { SmithEssence::EASY2_WEAPON }, 5 },
    { TR_RES_CURSE, { SmithEssence::RES_CURSE }, 10 },
    { TR_BERS_RAGE, {}, -1 },
    { TR_BRAND_MAGIC, { SmithEssence::BRAND_MAGIC }, 10 },
    { TR_IMPACT, {}, 0 },
    { TR_VUL_ACID, {}, -1 },
    { TR_VUL_COLD, {}, -1 },
    { TR_VUL_ELEC, {}, -1 },
    { TR_VUL_FIRE, {}, -1 },
    { TR_VUL_LITE, {}, -1 },
    { TR_IM_DARK, {}, 0 },
};

namespace {

template <typename T, typename... Args>
std::shared_ptr<ISmithInfo> make_info(SmithEffect effect, concptr name, SmithCategory category, std::vector<SmithEssence> need_essences, int consumption, Args &&...args)
{
    return std::make_shared<T>(effect, name, category, std::move(need_essences), consumption, std::forward<Args>(args)...);
}

std::shared_ptr<ISmithInfo> make_basic_smith_info(SmithEffect effect, concptr name, SmithCategory category, std::vector<SmithEssence> need_essences, int consumption, TrFlags add_flags)
{
    return make_info<BasicSmithInfo>(effect, name, category, std::move(need_essences), consumption, std::move(add_flags));
}

}

/*!
 * @brief 鍛冶情報テーブル
 */
const std::vector<std::shared_ptr<ISmithInfo>> Smith::smith_info_table = {
    make_basic_smith_info(SmithEffect::NONE, _("なし", "None"), SmithCategory::NONE, std::vector<SmithEssence>{ SmithEssence::NONE }, 0, {}),

    make_basic_smith_info(SmithEffect::STR, _("腕力", "strength"), SmithCategory::PVAL, { SmithEssence::STR }, 20, { TR_STR }),
    make_basic_smith_info(SmithEffect::INT, _("知能", "intelligence"), SmithCategory::PVAL, { SmithEssence::INT }, 20, { TR_INT }),
    make_basic_smith_info(SmithEffect::WIS, _("賢さ", "wisdom"), SmithCategory::PVAL, { SmithEssence::WIS }, 20, { TR_WIS }),
    make_basic_smith_info(SmithEffect::DEX, _("器用さ", "dexterity"), SmithCategory::PVAL, { SmithEssence::DEX }, 20, { TR_DEX }),
    make_basic_smith_info(SmithEffect::CON, _("耐久力", "constitution"), SmithCategory::PVAL, { SmithEssence::CON }, 20, { TR_CON }),
    make_basic_smith_info(SmithEffect::CHR, _("魅力", "charisma"), SmithCategory::PVAL, { SmithEssence::CHR }, 20, { TR_CHR }),

    make_basic_smith_info(SmithEffect::SUST_STR, _("腕力維持", "sustain strength"), SmithCategory::ABILITY, { SmithEssence::SUST_STATUS }, 15, { TR_SUST_STR }),
    make_basic_smith_info(SmithEffect::SUST_INT, _("知能維持", "sustain intelligence"), SmithCategory::ABILITY, { SmithEssence::SUST_STATUS }, 15, { TR_SUST_INT }),
    make_basic_smith_info(SmithEffect::SUST_WIS, _("賢さ維持", "sustain wisdom"), SmithCategory::ABILITY, { SmithEssence::SUST_STATUS }, 15, { TR_SUST_WIS }),
    make_basic_smith_info(SmithEffect::SUST_DEX, _("器用さ維持", "sustain dexterity"), SmithCategory::ABILITY, { SmithEssence::SUST_STATUS }, 15, { TR_SUST_DEX }),
    make_basic_smith_info(SmithEffect::SUST_CON, _("耐久力維持", "sustain constitution"), SmithCategory::ABILITY, { SmithEssence::SUST_STATUS }, 15, { TR_SUST_CON }),
    make_basic_smith_info(SmithEffect::SUST_CHR, _("魅力維持", "sustain charisma"), SmithCategory::ABILITY, { SmithEssence::SUST_STATUS }, 15, { TR_SUST_CHR }),

    make_basic_smith_info(SmithEffect::MAGIC_MASTERY, _("魔力支配", "magic mastery"), SmithCategory::PVAL, { SmithEssence::MAGIC_MASTERY }, 20, { TR_MAGIC_MASTERY }),
    make_basic_smith_info(SmithEffect::STEALTH, _("隠密", "stealth"), SmithCategory::PVAL, { SmithEssence::STEALTH }, 40, { TR_STEALTH }),
    make_basic_smith_info(SmithEffect::SEARCH, _("探索", "searching"), SmithCategory::PVAL, { SmithEssence::SEARCH }, 15, { TR_SEARCH }),
    make_basic_smith_info(SmithEffect::INFRA, _("赤外線視力", "infravision"), SmithCategory::PVAL, { SmithEssence::INFRA }, 15, { TR_INFRA }),
    make_basic_smith_info(SmithEffect::TUNNEL, _("採掘", "digging"), SmithCategory::PVAL, { SmithEssence::TUNNEL }, 15, { TR_TUNNEL }),
    make_basic_smith_info(SmithEffect::SPEED, _("スピード", "speed"), SmithCategory::PVAL, { SmithEssence::SPEED }, 12, { TR_SPEED }),
    make_basic_smith_info(SmithEffect::BLOWS, _("追加攻撃", "extra attack"), SmithCategory::WEAPON_ATTR, { SmithEssence::BLOWS }, 20, { TR_BLOWS }),

    make_basic_smith_info(SmithEffect::CHAOTIC, _("カオス攻撃", "chaos brand"), SmithCategory::WEAPON_ATTR, { SmithEssence::CHAOTIC }, 15, { TR_CHAOTIC }),
    make_basic_smith_info(SmithEffect::VAMPIRIC, _("吸血攻撃", "vampiric brand"), SmithCategory::WEAPON_ATTR, { SmithEssence::VAMPIRIC }, 60, { TR_VAMPIRIC }),
    make_basic_smith_info(SmithEffect::EARTHQUAKE, _("地震攻撃", "quake brand"), SmithCategory::WEAPON_ATTR, { SmithEssence::EATHQUAKE }, 15, { TR_EARTHQUAKE }),
    make_basic_smith_info(SmithEffect::BRAND_POIS, _("毒殺", "poison brand"), SmithCategory::WEAPON_ATTR, { SmithEssence::BRAND_POIS }, 20, { TR_BRAND_POIS }),
    make_basic_smith_info(SmithEffect::BRAND_ACID, _("溶解", "acid brand"), SmithCategory::WEAPON_ATTR, { SmithEssence::BRAND_ACID }, 20, { TR_BRAND_ACID }),
    make_basic_smith_info(SmithEffect::BRAND_ELEC, _("電撃", "electric brand"), SmithCategory::WEAPON_ATTR, { SmithEssence::BRAND_ELEC }, 20, { TR_BRAND_ELEC }),
    make_basic_smith_info(SmithEffect::BRAND_FIRE, _("焼棄", "fire brand"), SmithCategory::WEAPON_ATTR, { SmithEssence::BRAND_FIRE }, 20, { TR_BRAND_FIRE }),
    make_basic_smith_info(SmithEffect::BRAND_COLD, _("凍結", "cold brand"), SmithCategory::WEAPON_ATTR, { SmithEssence::BRAND_COLD }, 20, { TR_BRAND_COLD }),
    make_basic_smith_info(SmithEffect::VORPAL, _("切れ味", "sharpness"), SmithCategory::WEAPON_ATTR, { SmithEssence::BRAND_ACID, SmithEssence::BRAND_ELEC, SmithEssence::BRAND_FIRE, SmithEssence::BRAND_COLD }, 10, { TR_VORPAL }),
    make_basic_smith_info(SmithEffect::BRAND_MAGIC, _("魔術属性攻撃", "magic brand"), SmithCategory::WEAPON_ATTR, { SmithEssence::BRAND_MAGIC }, 20, { TR_BRAND_MAGIC }),
    make_basic_smith_info(SmithEffect::XTRA_MIGHT, _("射撃倍率強化", "enhance firing power"), SmithCategory::WEAPON_ATTR, { SmithEssence::STRENGTHEN_BOW }, 50, { TR_XTRA_MIGHT }),
    make_basic_smith_info(SmithEffect::XTRA_SHOTS, _("射撃速度強化", "enhance firing rate"), SmithCategory::WEAPON_ATTR, { SmithEssence::STRENGTHEN_BOW }, 50, { TR_XTRA_SHOTS }),

    make_basic_smith_info(SmithEffect::IM_ACID, _("酸免疫", "acid immunity"), SmithCategory::RESISTANCE, { SmithEssence::IMMUNITY, SmithEssence::RES_ACID }, 200, { TR_IM_ACID }),
    make_basic_smith_info(SmithEffect::IM_ELEC, _("電撃免疫", "electric immunity"), SmithCategory::RESISTANCE, { SmithEssence::IMMUNITY, SmithEssence::RES_ELEC }, 200, { TR_IM_ELEC }),
    make_basic_smith_info(SmithEffect::IM_FIRE, _("火炎免疫", "fire immunity"), SmithCategory::RESISTANCE, { SmithEssence::IMMUNITY, SmithEssence::RES_FIRE }, 200, { TR_IM_FIRE }),
    make_basic_smith_info(SmithEffect::IM_COLD, _("冷気免疫", "cold immunity"), SmithCategory::RESISTANCE, { SmithEssence::IMMUNITY, SmithEssence::RES_COLD }, 200, { TR_IM_COLD }),
    make_basic_smith_info(SmithEffect::REFLECT, _("反射", "reflection"), SmithCategory::RESISTANCE, { SmithEssence::REFLECT }, 20, { TR_REFLECT }),

    make_basic_smith_info(SmithEffect::RES_ACID, _("耐酸", "resistance to acid"), SmithCategory::RESISTANCE, { SmithEssence::RES_ACID }, 15, { TR_RES_ACID }),
    make_basic_smith_info(SmithEffect::RES_ELEC, _("耐電撃", "resistance to electric"), SmithCategory::RESISTANCE, { SmithEssence::RES_ELEC }, 15, { TR_RES_ELEC }),
    make_basic_smith_info(SmithEffect::RES_FIRE, _("耐火炎", "resistance to fire"), SmithCategory::RESISTANCE, { SmithEssence::RES_FIRE }, 15, { TR_RES_FIRE }),
    make_basic_smith_info(SmithEffect::RES_COLD, _("耐冷気", "resistance to cold"), SmithCategory::RESISTANCE, { SmithEssence::RES_COLD }, 15, { TR_RES_COLD }),
    make_basic_smith_info(SmithEffect::RES_POIS, _("耐毒", "resistance to poison"), SmithCategory::RESISTANCE, { SmithEssence::RES_POIS }, 25, { TR_RES_POIS }),
    make_basic_smith_info(SmithEffect::RES_FEAR, _("耐恐怖", "resistance to fear"), SmithCategory::RESISTANCE, { SmithEssence::RES_FEAR }, 20, { TR_RES_FEAR }),
    make_basic_smith_info(SmithEffect::RES_LITE, _("耐閃光", "resistance to light"), SmithCategory::RESISTANCE, { SmithEssence::RES_LITE }, 20, { TR_RES_LITE }),
    make_basic_smith_info(SmithEffect::RES_DARK, _("耐暗黒", "resistance to dark"), SmithCategory::RESISTANCE, { SmithEssence::RES_DARK }, 20, { TR_RES_DARK }),
    make_basic_smith_info(SmithEffect::RES_BLIND, _("耐盲目", "resistance to blind"), SmithCategory::RESISTANCE, { SmithEssence::RES_BLIND }, 20, { TR_RES_BLIND }),
    make_basic_smith_info(SmithEffect::RES_CONF, _("耐混乱", "resistance to confusion"), SmithCategory::RESISTANCE, { SmithEssence::RES_CONF }, 20, { TR_RES_CONF }),
    make_basic_smith_info(SmithEffect::RES_SOUND, _("耐轟音", "resistance to sound"), SmithCategory::RESISTANCE, { SmithEssence::RES_SOUND }, 20, { TR_RES_SOUND }),
    make_basic_smith_info(SmithEffect::RES_SHARDS, _("耐破片", "resistance to shard"), SmithCategory::RESISTANCE, { SmithEssence::RES_SHARDS }, 20, { TR_RES_SHARDS }),
    make_basic_smith_info(SmithEffect::RES_NETHER, _("耐地獄", "resistance to nether"), SmithCategory::RESISTANCE, { SmithEssence::RES_NETHER }, 20, { TR_RES_NETHER }),
    make_basic_smith_info(SmithEffect::RES_NEXUS, _("耐因果混乱", "resistance to nexus"), SmithCategory::RESISTANCE, { SmithEssence::RES_NEXUS }, 20, { TR_RES_NEXUS }),
    make_basic_smith_info(SmithEffect::RES_CHAOS, _("耐カオス", "resistance to chaos"), SmithCategory::RESISTANCE, { SmithEssence::RES_CHAOS }, 20, { TR_RES_CHAOS }),
    make_basic_smith_info(SmithEffect::RES_DISEN, _("耐劣化", "resistance to disenchantment"), SmithCategory::RESISTANCE, { SmithEssence::RES_DISEN }, 20, { TR_RES_DISEN }),
    make_basic_smith_info(SmithEffect::RES_WATER, _("耐水", "resistance to water"), SmithCategory::RESISTANCE, { SmithEssence::RES_WATER }, 20, { TR_RES_WATER }),
    make_basic_smith_info(SmithEffect::RES_TIME, _("耐時間逆転", "resistance to time"), SmithCategory::RESISTANCE, { SmithEssence::RES_TIME }, 20, { TR_RES_TIME }),
    make_basic_smith_info(SmithEffect::RES_CURSE, _("耐呪力", "resistance to curse"), SmithCategory::RESISTANCE, { SmithEssence::RES_CURSE }, 20, { TR_RES_CURSE }),

    make_basic_smith_info(SmithEffect::HOLD_EXP, _("経験値維持", "hold experience"), SmithCategory::ABILITY, { SmithEssence::HOLD_EXP }, 20, { TR_HOLD_EXP }),
    make_basic_smith_info(SmithEffect::FREE_ACT, _("麻痺知らず", "free action"), SmithCategory::ABILITY, { SmithEssence::FREE_ACT }, 20, { TR_FREE_ACT }),
    make_basic_smith_info(SmithEffect::WARNING, _("警告", "warning"), SmithCategory::ABILITY, { SmithEssence::WARNING }, 20, { TR_WARNING }),
    make_basic_smith_info(SmithEffect::LEVITATION, _("浮遊", "levitation"), SmithCategory::ABILITY, { SmithEssence::LEVITATION }, 20, { TR_LEVITATION }),
    make_basic_smith_info(SmithEffect::SEE_INVIS, _("可視透明", "see invisible"), SmithCategory::ABILITY, { SmithEssence::SEE_INVIS }, 20, { TR_SEE_INVIS }),
    make_basic_smith_info(SmithEffect::SLOW_DIGEST, _("遅消化", "slow digestion"), SmithCategory::ABILITY, { SmithEssence::SLOW_DIGEST }, 15, { TR_SLOW_DIGEST }),
    make_basic_smith_info(SmithEffect::REGEN, _("急速回復", "regeneration"), SmithCategory::ABILITY, { SmithEssence::REGEN }, 20, { TR_REGEN }),
    make_basic_smith_info(SmithEffect::TELEPORT, _("テレポート", "teleport"), SmithCategory::ABILITY, { SmithEssence::TELEPORT }, 25, { TR_TELEPORT }),
    make_basic_smith_info(SmithEffect::NO_MAGIC, _("反魔法", "anti magic"), SmithCategory::ABILITY, { SmithEssence::NO_MAGIC }, 15, { TR_NO_MAGIC }),
    make_basic_smith_info(SmithEffect::LITE, _("永久光源", "permanent light"), SmithCategory::ABILITY, { SmithEssence::LITE }, 15, { TR_LITE_1 }),
    make_basic_smith_info(SmithEffect::NO_TELE, _("反テレポート", "prevent teleportation"), SmithCategory::ABILITY, { SmithEssence::NO_TELE }, 15, { TR_NO_TELE }),

    make_basic_smith_info(SmithEffect::SLAY_EVIL, _("邪悪倍打", "slay evil"), SmithCategory::SLAYING, { SmithEssence::SLAY_EVIL }, 100, { TR_SLAY_EVIL }),
    make_basic_smith_info(SmithEffect::SLAY_ANIMAL, _("動物倍打", "slay animal"), SmithCategory::SLAYING, { SmithEssence::SLAY_ANIMAL }, 20, { TR_SLAY_ANIMAL }),
    make_basic_smith_info(SmithEffect::SLAY_UNDEAD, _("不死倍打", "slay undead"), SmithCategory::SLAYING, { SmithEssence::SLAY_UNDEAD }, 20, { TR_SLAY_UNDEAD }),
    make_basic_smith_info(SmithEffect::SLAY_DEMON, _("悪魔倍打", "slay demon"), SmithCategory::SLAYING, { SmithEssence::SLAY_DEMON }, 20, { TR_SLAY_DEMON }),
    make_basic_smith_info(SmithEffect::SLAY_ORC, _("オーク倍打", "slay orc"), SmithCategory::SLAYING, { SmithEssence::SLAY_ORC }, 20, { TR_SLAY_ORC }),
    make_basic_smith_info(SmithEffect::SLAY_TROLL, _("トロル倍打", "slay troll"), SmithCategory::SLAYING, { SmithEssence::SLAY_TROLL }, 20, { TR_SLAY_TROLL }),
    make_basic_smith_info(SmithEffect::SLAY_GIANT, _("巨人倍打", "slay giant"), SmithCategory::SLAYING, { SmithEssence::SLAY_GIANT }, 20, { TR_SLAY_GIANT }),
    make_basic_smith_info(SmithEffect::SLAY_DRAGON, _("竜倍打", "slay dragon"), SmithCategory::SLAYING, { SmithEssence::SLAY_DRAGON }, 20, { TR_SLAY_DRAGON }),
    make_basic_smith_info(SmithEffect::SLAY_HUMAN, _("人間倍打", "slay human"), SmithCategory::SLAYING, { SmithEssence::SLAY_HUMAN }, 20, { TR_SLAY_HUMAN }),
    make_basic_smith_info(SmithEffect::SLAY_GOOD, _("善良倍打", "slay good"), SmithCategory::SLAYING, { SmithEssence::SLAY_GOOD }, 20, { TR_SLAY_GOOD }),

    make_basic_smith_info(SmithEffect::KILL_EVIL, _("邪悪倍倍打", "kill evil"), SmithCategory::NONE, { SmithEssence::SLAY_EVIL }, 0, { TR_KILL_EVIL }), // 強力すぎるため無効(SmithCategory:NONE)
    make_basic_smith_info(SmithEffect::KILL_ANIMAL, _("動物倍倍打", "kill animal"), SmithCategory::SLAYING, { SmithEssence::SLAY_ANIMAL }, 60, { TR_KILL_ANIMAL }),
    make_basic_smith_info(SmithEffect::KILL_UNDEAD, _("不死倍倍打", "kill undead"), SmithCategory::SLAYING, { SmithEssence::SLAY_UNDEAD }, 60, { TR_KILL_UNDEAD }),
    make_basic_smith_info(SmithEffect::KILL_DEMON, _("悪魔倍倍打", "kill demon"), SmithCategory::SLAYING, { SmithEssence::SLAY_DEMON }, 60, { TR_KILL_DEMON }),
    make_basic_smith_info(SmithEffect::KILL_ORC, _("オーク倍倍打", "kill orc"), SmithCategory::SLAYING, { SmithEssence::SLAY_ORC }, 60, { TR_KILL_ORC }),
    make_basic_smith_info(SmithEffect::KILL_TROLL, _("トロル倍倍打", "kill troll"), SmithCategory::SLAYING, { SmithEssence::SLAY_TROLL }, 60, { TR_KILL_TROLL }),
    make_basic_smith_info(SmithEffect::KILL_GIANT, _("巨人倍倍打", "kill giant"), SmithCategory::SLAYING, { SmithEssence::SLAY_GIANT }, 60, { TR_KILL_GIANT }),
    make_basic_smith_info(SmithEffect::KILL_DRAGON, _("竜倍倍打", "kill dragon"), SmithCategory::SLAYING, { SmithEssence::SLAY_DRAGON }, 60, { TR_KILL_DRAGON }),
    make_basic_smith_info(SmithEffect::KILL_HUMAN, _("人間倍倍打", "kill human"), SmithCategory::SLAYING, { SmithEssence::SLAY_HUMAN }, 60, { TR_KILL_HUMAN }),
    make_basic_smith_info(SmithEffect::KILL_GOOD, _("善良倍倍打", "kill good"), SmithCategory::SLAYING, { SmithEssence::SLAY_GOOD }, 60, { TR_KILL_GOOD }),

    make_basic_smith_info(SmithEffect::TELEPATHY, _("テレパシー", "telepathy"), SmithCategory::ESP, { SmithEssence::TELEPATHY }, 15, { TR_TELEPATHY }),
    make_basic_smith_info(SmithEffect::ESP_ANIMAL, _("動物ESP", "sense animal"), SmithCategory::ESP, { SmithEssence::SLAY_ANIMAL }, 40, { TR_ESP_ANIMAL }),
    make_basic_smith_info(SmithEffect::ESP_UNDEAD, _("不死ESP", "sense undead"), SmithCategory::ESP, { SmithEssence::SLAY_UNDEAD }, 40, { TR_ESP_UNDEAD }),
    make_basic_smith_info(SmithEffect::ESP_DEMON, _("悪魔ESP", "sense demon"), SmithCategory::ESP, { SmithEssence::SLAY_DEMON }, 40, { TR_ESP_DEMON }),
    make_basic_smith_info(SmithEffect::ESP_ORC, _("オークESP", "sense orc"), SmithCategory::ESP, { SmithEssence::SLAY_ORC }, 40, { TR_ESP_ORC }),
    make_basic_smith_info(SmithEffect::ESP_TROLL, _("トロルESP", "sense troll"), SmithCategory::ESP, { SmithEssence::SLAY_TROLL }, 40, { TR_ESP_TROLL }),
    make_basic_smith_info(SmithEffect::ESP_GIANT, _("巨人ESP", "sense giant"), SmithCategory::ESP, { SmithEssence::SLAY_GIANT }, 40, { TR_ESP_GIANT }),
    make_basic_smith_info(SmithEffect::ESP_DRAGON, _("竜ESP", "sense dragon"), SmithCategory::ESP, { SmithEssence::SLAY_DRAGON }, 40, { TR_ESP_DRAGON }),
    make_basic_smith_info(SmithEffect::ESP_HUMAN, _("人間ESP", "sense human"), SmithCategory::ESP, { SmithEssence::SLAY_HUMAN }, 40, { TR_ESP_HUMAN }),
    make_basic_smith_info(SmithEffect::ESP_GOOD, _("善良ESP", "sense good"), SmithCategory::ESP, { SmithEssence::SLAY_GOOD }, 40, { TR_ESP_GOOD }),
    make_basic_smith_info(SmithEffect::ESP_UNIQUE, _("ユニークESP", "sense unique"), SmithCategory::ESP, { SmithEssence::UNIQUE }, 100, { TR_ESP_UNIQUE }),

    make_basic_smith_info(SmithEffect::SH_FIRE, _("火炎オーラ", "fiery sheath"), SmithCategory::ETC, { SmithEssence::RES_FIRE, SmithEssence::BRAND_FIRE }, 50, { TR_RES_FIRE, TR_SH_FIRE }),
    make_basic_smith_info(SmithEffect::SH_ELEC, _("電撃オーラ", "electric sheath"), SmithCategory::ETC, { SmithEssence::RES_ELEC, SmithEssence::BRAND_ELEC }, 50, { TR_RES_ELEC, TR_SH_ELEC }),
    make_basic_smith_info(SmithEffect::SH_COLD, _("冷気オーラ", "sheath of coldness"), SmithCategory::ETC, { SmithEssence::RES_COLD, SmithEssence::BRAND_COLD }, 50, { TR_RES_COLD, TR_SH_COLD }),

    make_basic_smith_info(SmithEffect::RESISTANCE, _("全耐性", "resistance"), SmithCategory::RESISTANCE, { SmithEssence::RES_ACID, SmithEssence::RES_ELEC, SmithEssence::RES_FIRE, SmithEssence::RES_COLD }, 150, { TR_RES_ACID, TR_RES_ELEC, TR_RES_FIRE, TR_RES_COLD }),
    make_info<SlayingGlovesSmithInfo>(SmithEffect::SLAY_GLOVE, _("殺戮の小手", "gauntlets of slaying"), SmithCategory::ETC, { SmithEssence::ATTACK }, 200),
    make_basic_smith_info(SmithEffect::EASY_2WEAPON, _("源氏の小手", "guantlets of Genji"), SmithCategory::ETC, { SmithEssence::EASY2_WEAPON }, 20, { TR_EASY2_WEAPON }),

    make_info<ActivationSmithInfo>(SmithEffect::ACT_QUAKE, _("地震", "quake"), SmithCategory::ACTIVATION, { SmithEssence::EATHQUAKE }, 15, RandomArtActType::QUAKE),
    make_info<ActivationSmithInfo>(SmithEffect::ACT_RES_ACID, _("酸耐性", "resist acid"), SmithCategory::ACTIVATION, { SmithEssence::RES_ACID }, 30, RandomArtActType::RESIST_ACID),
    make_info<ActivationSmithInfo>(SmithEffect::ACT_RES_ELEC, _("電撃耐性", "resist electricity"), SmithCategory::ACTIVATION, { SmithEssence::RES_ELEC }, 30, RandomArtActType::RESIST_ELEC),
    make_info<ActivationSmithInfo>(SmithEffect::ACT_RES_FIRE, _("火炎耐性", "resist fire"), SmithCategory::ACTIVATION, { SmithEssence::RES_FIRE }, 30, RandomArtActType::RESIST_FIRE),
    make_info<ActivationSmithInfo>(SmithEffect::ACT_RES_COLD, _("冷気耐性", "resist cold"), SmithCategory::ACTIVATION, { SmithEssence::RES_COLD }, 30, RandomArtActType::RESIST_COLD),
    make_info<ActivationSmithInfo>(SmithEffect::ACT_RES_POIS, _("毒耐性", "resist poison"), SmithCategory::ACTIVATION, { SmithEssence::RES_POIS }, 40, RandomArtActType::RESIST_POIS),
    make_info<ActivationSmithInfo>(SmithEffect::ACT_PHASE_DOOR, _("ショート・テレポート", "blink"), SmithCategory::ACTIVATION, { SmithEssence::TELEPORT }, 30, RandomArtActType::PHASE_DOOR),
    make_info<ActivationSmithInfo>(SmithEffect::ACT_TELEPORT, _("テレポート", "teleport"), SmithCategory::ACTIVATION, { SmithEssence::TELEPORT }, 40, RandomArtActType::TELEPORT),
    make_info<ActivationSmithInfo>(SmithEffect::ACT_SPEED, _("スピード", "speed"), SmithCategory::ACTIVATION, { SmithEssence::SPEED }, 25, RandomArtActType::SPEED),
    make_info<ActivationSmithInfo>(SmithEffect::ACT_STONE_MUD, _("岩石溶解", "stone to mud"), SmithCategory::ACTIVATION, { SmithEssence::TUNNEL }, 100, RandomArtActType::STONE_MUD),
    make_info<ActivationSmithInfo>(SmithEffect::ACT_LIGHT, _("イルミネーション", "light area"), SmithCategory::ACTIVATION, { SmithEssence::LITE }, 30, RandomArtActType::LIGHT),
    make_info<ActivationSmithInfo>(SmithEffect::ACT_MAP_LIGHT, _("魔法の地図と光", "light & map area"), SmithCategory::ACTIVATION, { SmithEssence::SEARCH, SmithEssence::LITE }, 30, RandomArtActType::MAP_LIGHT),
    make_info<ActivationSmithInfo>(SmithEffect::ACT_DETECT_ALL, _("全感知", "detection"), SmithCategory::ACTIVATION, { SmithEssence::SEARCH, SmithEssence::TELEPATHY }, 30, RandomArtActType::DETECT_ALL),
    make_info<ActivationSmithInfo>(SmithEffect::ACT_DETECT_UNIQUE, _("階にいるユニークモンスターを表示", "list of the uniques on the level"), SmithCategory::ACTIVATION, { SmithEssence::UNIQUE }, 100, RandomArtActType::DETECT_UNIQUE),
    make_info<ActivationSmithInfo>(SmithEffect::ACT_REST_EXP, _("経験値復活", "restore experience"), SmithCategory::ACTIVATION, { SmithEssence::HOLD_EXP }, 30, RandomArtActType::REST_EXP),
    make_info<ActivationSmithInfo>(SmithEffect::ACT_REST_ALL, _("全ステータスと経験値復活", "restore stats and experience"), SmithCategory::ACTIVATION, { SmithEssence::SUST_STATUS, SmithEssence::HOLD_EXP }, 40, RandomArtActType::REST_ALL),
    make_info<ActivationSmithInfo>(SmithEffect::ACT_PROT_EVIL, _("対邪悪結界", "protect evil"), SmithCategory::ACTIVATION, { SmithEssence::SLAY_EVIL }, 100, RandomArtActType::PROT_EVIL),
    make_info<ActivationSmithInfo>(SmithEffect::ACT_DISP_EVIL, _("邪悪退散", "dispel evil"), SmithCategory::ACTIVATION, { SmithEssence::SLAY_EVIL }, 100, RandomArtActType::DISP_EVIL),
    make_info<ActivationSmithInfo>(SmithEffect::ACT_DISP_GOOD, _("善良退散", "dispel good"), SmithCategory::ACTIVATION, { SmithEssence::SLAY_GOOD }, 50, RandomArtActType::DISP_GOOD),
    make_info<ActivationSmithInfo>(SmithEffect::ACT_PESTICIDE, _("害虫駆除", "dispel pests"), SmithCategory::ACTIVATION, { SmithEssence::BRAND_POIS }, 30, RandomArtActType::PESTICIDE),
    make_info<ActivationSmithInfo>(SmithEffect::ACT_BA_ACID, _("アシッド・ボール", "ball of acid"), SmithCategory::ACTIVATION, { SmithEssence::BRAND_ACID }, 40, RandomArtActType::BA_ACID_1),
    make_info<ActivationSmithInfo>(SmithEffect::ACT_BA_ELEC, _("サンダー・ボール", "ball of lightning"), SmithCategory::ACTIVATION, { SmithEssence::BRAND_ELEC }, 40, RandomArtActType::BA_ELEC_2),
    make_info<ActivationSmithInfo>(SmithEffect::ACT_BA_FIRE, _("ファイア・ボール", "ball of fire"), SmithCategory::ACTIVATION, { SmithEssence::BRAND_FIRE }, 40, RandomArtActType::BA_FIRE_4),
    make_info<ActivationSmithInfo>(SmithEffect::ACT_BA_COLD, _("アイス・ボール", "ball of cold"), SmithCategory::ACTIVATION, { SmithEssence::BRAND_COLD }, 40, RandomArtActType::BA_COLD_2),
    make_info<ActivationSmithInfo>(SmithEffect::ACT_BA_NUKE, _("放射能球", "ball of nuke"), SmithCategory::ACTIVATION, { SmithEssence::BRAND_POIS, SmithEssence::CHAOTIC }, 30, RandomArtActType::BA_NUKE_1),
    make_info<ActivationSmithInfo>(SmithEffect::ACT_SUNLIGHT, _("太陽光線", "beam of sunlight"), SmithCategory::ACTIVATION, { SmithEssence::LITE }, 40, RandomArtActType::SUNLIGHT),
    make_info<ActivationSmithInfo>(SmithEffect::ACT_DRAIN, _("吸血の矢", "drain bolt"), SmithCategory::ACTIVATION, { SmithEssence::VAMPIRIC }, 100, RandomArtActType::DRAIN_1),
    make_info<ActivationSmithInfo>(SmithEffect::ACT_CONFUSE, _("パニック・モンスター", "confuse monster"), SmithCategory::ACTIVATION, { SmithEssence::CHAOTIC }, 30, RandomArtActType::CONFUSE),
    make_info<ActivationSmithInfo>(SmithEffect::ACT_SATIATE, _("空腹充足", "satisfy hunger"), SmithCategory::ACTIVATION, { SmithEssence::SLOW_DIGEST }, 50, RandomArtActType::SATIATE),
    make_info<ActivationSmithInfo>(SmithEffect::ACT_CURE_700, _("体力回復", "heal"), SmithCategory::ACTIVATION, { SmithEssence::REGEN }, 100, RandomArtActType::CURE_700),

    make_info<EnchantWeaponSmithInfo>(SmithEffect::ATTACK, _("攻撃", "weapon enchant"), SmithCategory::ENCHANT, { SmithEssence::ATTACK }, 30),
    make_info<EnchantArmourSmithInfo>(SmithEffect::AC, _("防御", "armor enchant"), SmithCategory::ENCHANT, { SmithEssence::AC }, 15),
    make_info<SustainSmithInfo>(SmithEffect::SUSTAIN, _("装備保持", "elements proof"), SmithCategory::ENCHANT, { SmithEssence::RES_ACID, SmithEssence::RES_ELEC, SmithEssence::RES_FIRE, SmithEssence::RES_COLD }, 10),
};
